{ config, pkgs, ... }:
{
  imports =
    [
      ./hardware-configuration.nix
      ./host-specific.nix
      ../../modules/home-manager.nix
      ../../modules/nur-and-unstable.nix
      ../../modules/unfree.nix
      ../../modules/base.nix
      ../../modules/base-efi.nix
      ../../modules/headless.nix

      ../../modules/ssh-fde-unlock.nix
      #../../modules/git.nix
      ../../modules/zsh.nix
      #../../modules/ssh-client.nix
      ../../modules/ssh-server.nix

      #../../modules/podman.nix
      ../../modules/docker.nix
      #../../modules/docker-pushrm.nix
      ../../modules/ContainerImages.nix

      ../../modules/sendmail-mta.nix
      ../../modules/journalwatch.nix

      ../../modules/btrbk
      ./btrbk-config.nix

      ../../modules/netcup-dns.nix
      ../../modules/de-p1st-monitor.nix
      ../../modules/spin-down/hdparm.nix
      ../../modules/spin-down/hd-idle.nix

      ../../modules/btrfs-scrub.nix
      ../../modules/btrfs-mount-options.nix
    ];

  networking.hostName = "yodaNas";
  boot.initrd.luks.devices."luks-3d974bd0-f373-469b-8e9c-2d5516e9f0f5".allowDiscards = true;
  yoda.btrfsFileSystems = ["/" "/mnt/data" "/mnt/backup"];
  #yoda.btrfsMounts = yoda.btrfsFileSystems;
  yoda.spin-down.hdparm = [
    # 4tb1
    "ata-WDC_WD40EFRX-68N32N0_WD-WCC7K0CPF0N1"
    # 3tb1
    "ata-WDC_WD30EFRX-68EUZN0_WD-WCC4N1173157"
    # 3tb2
    "ata-WDC_WD30EFRX-68EUZN0_WD-WMC4N0564095"
  ];
  yoda.spin-down.hd-idle = [];
  yoda.de-p1st-monitor = (builtins.readFile ../../assets/de-p1st-monitor/yodaNas.ini);

  boot.kernelParams = [
    # Microarchitectural Data Sampling (MDS), see https://www.kernel.org/doc/html/latest/admin-guide/hw-vuln/mds.html#mitigation-control-on-the-kernel-command-line
    "mds=full,nosmt"
  ];
  boot.kernelPackages = pkgs.linuxPackages;

  # Most users should never change this value after the initial install, for any reason, even if you've upgraded your system to a new NixOS release.
  system.stateVersion = "23.05";
}
