{ config, pkgs, lib, ... }:
{
  # NixOS general:
  #   https://github.com/NixOS/nixpkgs/tree/nixos-23.11/pkgs/games/
  #   https://nixos.wiki/wiki/Games
  #   https://nixos.wiki/wiki/Minetest_Server
  #   Multi player server for Factorio. https://nixos.wiki/wiki/Factorio
  #   Unreal Engine 4 Linux demos. https://search.nixos.org/packages?channel=23.11&from=0&size=50&sort=relevance&type=packages&query=ue4demos
  # General:
  #   https://wiki.archlinux.org/title/List_of_games
  #   https://en.wikipedia.org/wiki/List_of_open-source_video_games
  #   Current state of SimRacing in Linux (Updated to 2022-2). https://www.reddit.com/r/simracing/comments/siktd6/current_state_of_simracing_in_linux_updated_to/
  #   The Best Driving Games on Linux (2012). https://linuxaria.com/article/the-best-driving-games-and-car-games-on-linux
  # Game Launchers:
  #   https://nixos.wiki/wiki/Steam
  #   https://nixos.wiki/wiki/Lutris
  # Non-packaged games:
  #   https://ya2.itch.io/yorg (Racing game)
  #   -> However, there is `itch` a laouncher for ithc.io games
  #   https://unknown-horizons.org/ (A 2D realtime strategy simulation with an emphasis on economy and city building.)
  #   https://github.com/mgerdes/Open-Golf

  allowUnfree = [
    "SpaceCadetPinball"
    "osu-lazer-bin"
    "warsow"
    "assaultcube"
    "ut2004-demo"
  ];

  users.users.yoda = {
    packages = with pkgs; [
      #
      # Sandbox
      #

      # Infinite-world block sandbox game.
      # Mods and games for Minetest:
      # - Mineclonia, a fork of MineClone 2.
      minetest
      classicube # A lightweight, custom Minecraft Classic/ClassiCube client with optional additions written from scratch in C
      voxelands # Voxelands is a sandbox construction game based on Minetest
      airshipper # Provides automatic updates for the voxel RPG Veloren. https://www.veloren.net/

      #
      # Chess
      #
      gnome.gnome-chess # Play the classic two-player boardgame of chess
      gnuchess # GNU Chess engine. Can be used by gnome-chess.

      #
      # Mines
      #
      gnome.gnome-mines # Clear hidden mines from a minefield.
      libremines

      #
      # Arcade and puzzles
      #
      space-cadet-pinball # Reverse engineering of "3D Pinball for Windows – Space Cadet", a game bundled with Windows
      superTux # Classic 2D jump'n run sidescroller game
      mari0 # Crossover between Super Mario Bros. and Portal
      armagetronad # A multiplayer networked arcade racing game in 3D similar to Tron
      neverball # Tilt the floor to roll a ball
      gnujump # 2D jump game
      gnome.gnome-nibbles # Guide a worm around a maze
      gnome.gnome-taquin # Move tiles so that they reach their places
      gnome.gnome-tetravex # Complete the puzzle by matching numbered tiles
      gnugo # GNU Go - A computer go player
      sweethome3d.application # Design and visualize your future home. Does not start as of 2024-01-21, see https://github.com/NixOS/nixpkgs/issues/270010

      #
      # Music
      #
      osu-lazer-bin # Dance and rythm game
      stepmania # Dance and rhythm game

      #
      # First Person Shooter
      #
      warsow # Multiplayer FPS game designed for competitive gaming. Similar to Unreal Tournament.
      #warfork # Fork of warsow. Available on Steam: https://store.steampowered.com/app/671610/Warfork/. https://github.com/TeamForbiddenLLC/warfork-qfusion
      sauerbraten # Cube 2: Sauerbraten. A multiplayer & singleplayer first person shooter, the successor of the Cube FPS
      assaultcube # Fast and fun first-person-shooter based on the Cube FPS
      xonotic # The Fast Arena Shooter
      ut2004demo # Demo of Unreal Tournament 2004
      openspades # Voxel shooter. Compatible with Ace of Spades 0.75
      #
      # Third Person / 2D Shooter
      #
      teeworlds # Retro multiplayer shooter game

      #
      # RPG
      #
      freedroidrpg # Similar to Diablo. 3D with fixed camera angle.

      #
      # Strategy, Real-Time Strategy (RTS)
      #
      zeroadPackages.zeroad-unwrapped # 0 A.D. (pronounced “zero-ey-dee”) is a historical RTS game
      openra # OpenRA: recreates and modernizes classic RTS games, like Red Alert, Command & Conquer, and Dune 2000.
      unvanquished # A fast paced, first person strategy game
      megaglest # MegaGlest. Fork of Glest. RTS. Screnshots: https://megaglest.org/media/
      scorched3d # 3D Clone of the classic Scorched Earth. Turn-based artillery game and also a RTS game
      wesnoth # The Battle for Wesnoth, a turn-based strategy game with a fantasy theme. Similar to Heroes of Might and Magic
      hedgewars # Turn-based strategy artillery game similar to Worms
      hase # Hase is a gravity based artillery shooter. It is similar to Worms, Hedgewars or artillery, but the gravity force and direction depends on the mass nearby. It is optimized for mobile game consoles like the GP2X, Open Pandora or GCW Zero, so it doesn't make use of the mouse even on the PC.
      lincity_ng # A little bit more than SimCity 2000. Does not start, see https://github.com/NixOS/nixpkgs/issues/282687
      #openttd # Clone of the Microprose game "Transport Tycoon Deluxe"
      widelands # A real-time build-up strategy game. https://codeberg.org/wl/widelands

      #
      # Racing / Flight
      #
      torcs # The Open Racing Car Simulator (TORCS). Car racing game
      #speed_dreams # Car racing game - TORCS fork with more experimental approach. Does not compile (error: boost::bad_format_string: format-string is ill-formed). Unrelated pull request: https://github.com/NixOS/nixpkgs/pull/270194. Is available on flathub: https://flathub.org/apps/org.speed_dreams.SpeedDreams
      #vdrift-bin # VDrift. Car racing game. Does not start (missing game data?): https://github.com/NixOS/nixpkgs/issues/282647
      vdrift # VDrift
      stuntrally # Stunt Rally. Game with Track Editor. Based on VDrift and OGRE. https://stuntrally.tuxfamily.org/
      # Trigger Rally. Fast-paced single-player rally racing game. Flathub: https://flathub.org/apps/io.sourceforge.trigger_rally.TriggerRally
      flightgear # Flight simulator
      #
      # Arcade racing
      #
      superTuxKart # A 3D kart racing game
      extremetuxracer # High speed arctic racing game based on Tux Racer

      #crossfire-client # GTKv2 client for the Crossfire free MMORPG. Screenshot: https://crossfire.real-time.com/images/clients/debian10-screenshot.png

      #
      # Purchase/Game files required
      #
      #fallout1-ce
      #fallout2-ce # A fully working re-implementation of Fallout 2, with the same original gameplay, engine bugfixes, and some quality of life improvements
      #openrw # Unofficial recreation of the classic Grand Theft Auto III game executable. Flatpak: https://flathub.org/apps/org.openmw.OpenMW
      # SeriousSamClassic-VK (runs on Vulkan or SDL2), flatpak: https://flathub.org/apps/io.itch.tx00100xt.SeriousSamClassic-VK
      # Albion Online. On April 10, 2019, Albion Online went Free to Play. Players can buy premium with in-game currency (gold & silver). Flatpak: https://flathub.org/apps/com.albiononline.AlbionOnline
      # Vintage Story. To play this game, you must purchase an account at https://www.vintagestory.at/. Flathub: https://flathub.org/apps/at.vintagestory.VintageStory
    ];
  };
}
